/*  IPTalk - A P2P Communications System for IP Networks
		Version 1.0
		(C) 2001 Kevin H. Patterson
		IPTalk Version 1.0 is distributed under the GNU General Public License.
		Please see the included file "GPL License.txt" for details.
*/

// This class is my current lame attempt to implement a multi-threaded,
// event-driven derivative of the BNetEndpoint class. Improvements are
// more than welcome. (think 'Windows Winsock Control', only for real :)

// OK, I'll admit, the built-in alerts are *totally* lame.

#include "NetEndpointA.h"

NetEndpointA::NetEndpointA(int Protocol = SOCK_STREAM)
:
	BNetEndpoint(Protocol)
{
	Target = NULL;
	Listening = false;
	Receiving = false;
	ShowAlerts = true;
	if (BNetEndpoint::InitCheck() == B_ERROR)
		AlertInitCheck();
}

NetEndpointA::NetEndpointA(const BNetEndpoint &NetEndpoint)
:
	BNetEndpoint(NetEndpoint)
{
	Target = NULL;
	Listening = false;
	Receiving = false;
	ShowAlerts = true;
	if (BNetEndpoint::InitCheck() == B_ERROR)
		AlertInitCheck();
}

NetEndpointA::NetEndpointA(BMessage * Archive)
:
	BNetEndpoint(Archive)
{
	Target = NULL;
	Listening = false;
	Receiving = false;
	ShowAlerts = true;
	if (BNetEndpoint::InitCheck() == B_ERROR)
		AlertInitCheck();
}

BString NetEndpointA::LocalHost()
{
	char *Address[257];
	BString Result("unknown");
	if (BNetEndpoint::LocalAddr().GetAddr((char*)Address, NULL) == B_ERROR)
		AlertResolveLocalAddress();		
	else
		Result = (char*)Address;
	return Result;
}

uint16 NetEndpointA::LocalPort()
{
	uint16 Result;
	if (BNetEndpoint::LocalAddr().GetAddr(NULL, &Result) == B_ERROR)
	{
		AlertResolveLocalAddress();
		Result = 0;
	}
	return Result;
}
		
BString NetEndpointA::RemoteHost()
{
	char *Address[257];
	BString Result("unknown");
	if (BNetEndpoint::RemoteAddr().GetAddr((char*)Address, NULL) == B_ERROR)
		AlertResolveRemoteAddress();
	else	
		Result = (char*)Address;
	return Result;
}

uint16 NetEndpointA::RemotePort()
{
	uint16 Result;
	if (BNetEndpoint::RemoteAddr().GetAddr(NULL, &Result) == B_ERROR)
	{
		AlertResolveRemoteAddress();
		Result = 0;
	}
	return Result;
}

int32 ListenerThread(void * Data)
{
	NetEndpointA * ListenerEndpoint = (NetEndpointA *)Data;
	while (ListenerEndpoint->Listening)
	{
		BNetEndpoint * NewConnection = ListenerEndpoint->Accept();
		if (NewConnection)
		{
		  BMessage * MyMessage = new BMessage('InCn');
		  MyMessage->AddPointer("connection", NewConnection);
		  ListenerEndpoint->Target->PostMessage(MyMessage);
		  delete MyMessage;	
		}	
	}
	return true;
}

bool NetEndpointA::StartListening()
{
	if ((Target != NULL) & !Listening)
	{
		ListenerThreadID = spawn_thread(ListenerThread, "NEASocketListener", B_NORMAL_PRIORITY, (void *)this);
		if (ListenerThreadID >= 0)
		{
			Listening = true;
			resume_thread(ListenerThreadID);
			return true;
		}
	}
	ListenerThreadID = -1;
	return false;
}

bool NetEndpointA::StopListening()
{
	Listening = false;
	if (ListenerThreadID >= 0)
	{
		status_t * ExitValue = 0;
		wait_for_thread(ListenerThreadID, ExitValue);
		ListenerThreadID = -1;
	}
	return true;
}

int32 ReceiverThread(void * Data)
{
	static uint8 DataBuffer[65536];
	static int32 BytesReceived;
	NetEndpointA * ReceiverEndpoint = (NetEndpointA *)Data;
	while (ReceiverEndpoint->Receiving)
	{
		//we do this Lock(); Unlock() bit to wait for any ongoing socket operations
		ReceiverEndpoint->Locker.Lock();
		ReceiverEndpoint->Locker.Unlock();
		BytesReceived = ReceiverEndpoint->Receive(DataBuffer, 65536);
		if (BytesReceived > 0)
		{
		  BMessage * MyMessage = new BMessage('DRcv');
		  MyMessage->AddData("data", B_RAW_TYPE, DataBuffer, BytesReceived);
		  MyMessage->AddInt32("packetlen", BytesReceived);
		  ReceiverEndpoint->Target->PostMessage(MyMessage);
		  delete MyMessage;	
		}
		else
		{
			switch (ReceiverEndpoint->Error())
			{
			case EINTR:
			  break;
			case B_NO_ERROR:
				break;
			default:
				{
					BString ErrMsg = ReceiverEndpoint->ErrorStr();
					ErrMsg << "\12Error number " << ReceiverEndpoint->Error();
					BAlert * MyAlert = new BAlert(
						"Network Error",
						 ErrMsg.String(),
						"Unbelievable.",
						NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
						B_INFO_ALERT
					);
					MyAlert->Go();
				}
				return B_ERROR;
				break;
			}
		}	
	}
	return B_OK;
}

bool NetEndpointA::StartReceiving()
{
	if ((Target != NULL) & !Receiving)
	{
		ReceiverThreadID = spawn_thread(ReceiverThread, "NEASocketReceiver", B_NORMAL_PRIORITY, (void *)this);
		if (ReceiverThreadID >= 0)
		{
			Receiving = true;
			resume_thread(ReceiverThreadID);
			return true;
		}
	}
	ReceiverThreadID = -1;
	return false;
}

bool NetEndpointA::StopReceiving()
{
	Receiving = false;
	if (ReceiverThreadID >= 0)
	{
		status_t * ExitValue = 0;
		wait_for_thread(ReceiverThreadID, ExitValue);
		ReceiverThreadID = -1;
	}
	return true;
}

int32 NetEndpointA::SendA(const void * buffer, size_t size, int flags = 0)
{
	int32 Result = B_ERROR;
	Locker.Lock();
		Result = BNetEndpoint::Send(buffer, size, flags);
	Locker.Unlock();
	return Result;
}

void NetEndpointA::AlertInitCheck()
{
	if (ShowAlerts)
	{
		BAlert * MyAlert = new BAlert(
			"Network Error",
			"A network interface could not be initialized.\12BNetEndpoint::InitCheck()\12\12Try restarting this application; \12if that doesn't help, try restarting net_server.",
			"Doh!",
			NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
			B_WARNING_ALERT
		);
		MyAlert->Go();
	}
}

void NetEndpointA::AlertResolveLocalAddress()
{
	if (ShowAlerts)
	{
		BAlert * MyAlert = new BAlert(
			"Network Error",
			"The address of a local network interface could not be resolved.\12BNetEndpoint::LocalAddr()::GetAddr()\12\12Sorry, I have absolutely no clue why this happened.",
			"Unbelievable.",
			NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
			B_WARNING_ALERT
		);
		MyAlert->Go();
	}
}

void NetEndpointA::AlertResolveRemoteAddress()
{
	if (ShowAlerts)
	{
		BAlert * MyAlert = new BAlert(
			"Network Error",
			"The address of a remote network interface could not be resolved.\12BNetEndpoint::LocalAddr()::GetAddr()\12\12Sorry, I have absolutely no clue why this happened.",
			"Unbelievable.",
			NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
			B_WARNING_ALERT
		);
		MyAlert->Go();
	}
}
