/*  IPTalk - A P2P Communications System for IP Networks
		Version 1.0
		(C) 2001 Kevin H. Patterson
		IPTalk Version 1.0 is distributed under the GNU General Public License.
		Please see the included file "GPL License.txt" for details.
*/

#include "clsChatWindow.h"
#include "PacketStrucs.h"

clsChatWindow::clsChatWindow()
:
	BWindow(BRect(100,100,410,400), "Chat Session", B_TITLED_WINDOW, B_WILL_DRAW)
{
	ConnectionState = StateDisconnected;
	Connection = new NetEndpointA(SOCK_STREAM);
	Connection->Target = this;
	if (Connection->InitCheck() != B_OK)
		this->PostMessage(B_QUIT_REQUESTED);
  SetupViews();
	OutText->Connected = true;
}

clsChatWindow::clsChatWindow(BNetEndpoint * NewConnection)
:
	BWindow(BRect(200,200,510,500), "Chat Session", B_TITLED_WINDOW, B_WILL_DRAW)
{
	ConnectionState = StateDisconnected;
	Connection = new NetEndpointA(*NewConnection);
	Connection->Target = this;
	if (Connection->InitCheck() != B_OK)
		this->PostMessage(B_QUIT_REQUESTED);
  SetupViews();
  ConnectionState = StateConnecting;
  TransitionState(StateConnected);
  {
  	char *Address[257];
		if (Connection->RemoteAddr().GetAddr((char*)Address, NULL) == B_ERROR)
			ConnectIP->SetText("unknown");
		else
			ConnectIP->SetText((char*)Address);
	}
  {
  	uint16 Port;
		if (Connection->RemoteAddr().GetAddr(NULL, &Port) == B_ERROR)
			ConnectPort->SetText("0");
		else
		{
  		BString PortText = "";
  		PortText << int(Port);
			ConnectPort->SetText(PortText.String());
		}
	}
	OutText->Connected = true;
}

bool clsChatWindow::SetupViews()
{
	MainView = new BView(BWindow::Bounds(), "MainView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS);
	MainView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	BWindow::AddChild(MainView);

		TopClip = 32;
		ConnectionView = new BView(BRect(0, 0, MainView->Bounds().Width(), TopClip - 1), "ConnectionView", B_FOLLOW_TOP | B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW | B_FRAME_EVENTS);
		ConnectionView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		
			ConnectIP = new BTextControl(BRect(5, 5, 10, 10), "ConnectIP", "Connect to:", "127.0.0.1", NULL);
			ConnectIP->ResizeToPreferred();
			float MaxHeight = ConnectIP->Bounds().Height();
			
			ConnectPort = new BTextControl(BRect(5, 5, 10, 10), "ConnectIP", "Port:", "5038", NULL);
			ConnectPort->ResizeToPreferred();
			if (ConnectPort->Bounds().Height() > MaxHeight)
			{	MaxHeight = ConnectPort->Bounds().Height();	}
  		
			ConnectButton = new BButton(BRect(5, 5, 10, 10), "ConnectButton", "Connect", new BMessage('Cnct'), B_FOLLOW_TOP | B_FOLLOW_RIGHT);
			ConnectButton->ResizeToPreferred();
			if (ConnectButton->Bounds().Height() > MaxHeight)
			{	MaxHeight = ConnectButton->Bounds().Height();	}
			MaxHeight /= 2;
			MaxHeight += 4;
			
			ConnectIP->MoveTo(5, MaxHeight - ConnectIP->Bounds().Height() / 2);
			ConnectPort->MoveTo(ConnectIP->Bounds().right + 10, MaxHeight - ConnectPort->Bounds().Height() / 2);
			ConnectButton->MoveTo(ConnectionView->Bounds().right - ConnectButton->Bounds().Width() - 5, MaxHeight - ConnectButton->Bounds().Height() / 2);
			
		ConnectionView->AddChild(ConnectIP);
		ConnectionView->AddChild(ConnectPort);
		ConnectionView->AddChild(ConnectButton);
	
		ConnectionView->ResizeTo(MainView->Bounds().Width(), MaxHeight * 2 - 4);
		TopClip = ConnectionView->Bounds().Height() + 1;

		MainView->AddChild(ConnectionView);

	int32 ViewHeight = int((MainView->Bounds().Height() - TopClip) / 2 - 0.5);
  	
	InTextView = new BView(BRect(0, TopClip, MainView->Bounds().Width(), ViewHeight + TopClip), "InTextView", B_FOLLOW_NONE, B_WILL_DRAW | B_FRAME_EVENTS);
	InTextView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	MainView->AddChild(InTextView);

	OutTextView = new BView(BRect(0, MainView->Bounds().Height() - ViewHeight, MainView->Bounds().Width(), MainView->Bounds().Height()), "OutTextView", B_FOLLOW_NONE, B_WILL_DRAW | B_FRAME_EVENTS);
	OutTextView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
  MainView->AddChild(OutTextView);
 
	InText = new clsConnectedTextView(BRect(5,5,40,40), "InText", BRect(5,5,30,30), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE);
	InText->ChatWindow = this;
	InText->MakeEditable(false);
 	{
 		BScrollView * InTextScroll = new BScrollView("InTextScroll", InText, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS, false, true);
	  InTextView->AddChild(InTextScroll);
		InTextScroll->MoveTo(5,5);
		InTextScroll->ResizeTo(InTextView->Bounds().Width() - 10, InTextView->Bounds().Height() - 10);
	}
  	
	OutText = new clsConnectedTextView(BRect(5,5,40,40), "OutText", BRect(5,5,30,30), B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE);
	OutText->ChatWindow = this;
	OutText->MakeEditable(false);
  {
  	BScrollView * OutTextScroll = new BScrollView("OutTextScroll", OutText, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS, false, true);
		OutTextView->AddChild(OutTextScroll);
		OutTextScroll->MoveTo(5,5);
		OutTextScroll->ResizeTo(OutTextView->Bounds().Width() - 10, OutTextView->Bounds().Height() - 10);
	}
	
	clsChatWindow::FrameResized(BWindow::Bounds().Width(), BWindow::Bounds().Height());
	return true;
}

bool clsChatWindow::QuitRequested()
{
	TransitionState(StateDisconnected);
	return BWindow::QuitRequested();
}

void clsChatWindow::MessageReceived(BMessage* Message)
{
	static PacketMulti Packet;
	switch (Message->what)
	{
	case 'TIns':
  	{
  		int32 offset = 0;
  		int32 length = 0;
  		BString text = "";
  		Message->FindInt32("offset", 0, &offset);
  		Message->FindInt32("length", 0, &length);  		
			Message->FindString("text", 0, &text);
  		Packet.Magic = 0xFCFE;
  		Packet.Type = 3;
  		Packet.SubType = 1;
  		Packet.Data.TIns.Offset = offset;
  		Packet.Data.TIns.Length = length;
  		text.CopyInto((char *)&Packet.Data.TIns.Text, 0, text.CountChars());
  		Packet.Length = 14 + length;
			Connection->SendA(&Packet, Packet.Length);
		}
		break;
	case 'TDel':
		{
			int32 start = 0;
			int32 finish = 0;
			Message->FindInt32("start", 0, &start);
			Message->FindInt32("finish", 0, &finish);
			Packet.Magic = 0xFCFE;
			Packet.Type = 3;
			Packet.SubType = 2;
			Packet.Data.TDel.Start = start;
			Packet.Data.TDel.Finish = finish;
			Packet.Length = 14;
			Connection->SendA(&Packet, Packet.Length);
		}
		break;
	case 'DRcv':
		{
			uint8 * DataBuffer = NULL;
			ssize_t DataLength = 0;
			if (Message->FindData("data", B_ANY_TYPE, 0, (const void **)&DataBuffer, &DataLength) != B_OK)
			{
				BAlert * MyAlert = new BAlert(
					"Data Parsing Error",
					"BMessage::FindData('data') failed in DRcv message from socket.",
					"Doh!",
					NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
					B_WARNING_ALERT
				);
				MyAlert->Go();
			}
			else
			{
				uint32 DataOffset = 0;
				while (DataOffset < (uint32)DataLength)
				{
					PacketMulti * InPacket = reinterpret_cast<PacketMulti *>(&DataBuffer[DataOffset]);
					if (InPacket->Magic != 0xFCFE)
					{
						BAlert * MyAlert = new BAlert(
							"Data Parsing Error",
							"Unrecognized packet received.",
							"Doh!",
							NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
							B_WARNING_ALERT
						);
						MyAlert->Go();
						break;
					}
					else
					{
						switch (InPacket->Type)
						{
						case 0:
							switch (InPacket->SubType)
							{
							case 1:
								TransitionState(StateDisconnected);
								{
									BAlert * MyAlert = new BAlert(
									"IPTalk Chat Session",
										"The remote host has closed the connection.",
										"Bye.",
										NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
										B_INFO_ALERT
									);
									MyAlert->Go();
								}
								break;
							}
							break;
						case 3:
							switch (InPacket->SubType)
							{
							case 1:
								InText->Insert(InPacket->Data.TIns.Offset, (const char *)InPacket->Data.TIns.Text, InPacket->Data.TIns.Length);
								InText->ScrollToOffset(InPacket->Data.TIns.Offset + InPacket->Data.TIns.Length);
								break;
							case 2:
								InText->Delete(InPacket->Data.TDel.Start, InPacket->Data.TDel.Finish);
								break;
							}
							break;
						}
					}
					DataOffset += (ssize_t)InPacket->Length;
				}
			}
		}	
		break;
	case 'Cnct':
		{
			clsChatWindow::TransitionState();
		}
		break;
	default:
		{
	  	BWindow::MessageReceived(Message);
	  }
	  break;
	}
}

bool clsChatWindow::TransitionState(int32 NewState = StateAuto)
{
	switch (ConnectionState)
	{
	case StateDisconnected:
		if (NewState == StateConnecting || NewState == StateAuto)
		{
  		ConnectButton->SetLabel("Cancel");
  		InText->SelectAll();
			InText->Clear();
			OutText->SelectAll();
			OutText->Clear();
			ConnectionState = StateConnecting;
  		BWindow::Sync();
  		if (Connection->Connect(ConnectIP->Text(), atoi(ConnectPort->Text())) != B_OK)
			{
				BAlert * MyAlert = new BAlert(
					"Connection Error",
					"The connection could not be established.\12BNetEndpoint::Connect()\12\12Please make sure the destination address is reachable on your network.\12(Like, dial up your ISP...)",
					"Stupid Me.",
					NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
					B_WARNING_ALERT
				);
				MyAlert->Go();
				ConnectionState = StateDisconnected;
			}
			else
			{
				ConnectionState = StateConnected;
				Connection->StartReceiving();
			}  
  	}
   	break;
  case StateConnected:
		if (NewState == StateDisconnected || NewState == StateAuto)
		{
	    ConnectionState = StateDisconnected;
			Connection->StopReceiving();
			
			PacketMulti Packet;
			Packet.Magic = 0xFCFE;
			Packet.Type = 0;
			Packet.SubType = 1;
			Packet.Length = 6;
			Connection->SendA(&Packet, Packet.Length);
			
	    Connection->Close();
  	}
    break;
  case StateConnecting:
		if (NewState == StateDisconnected || NewState == StateAuto)
	    ConnectionState = StateDisconnected;
		if (NewState == StateConnected)
		{
	    ConnectionState = StateConnected;
			Connection->StartReceiving();
	  }		   
    break;
	}
	if (ConnectionState == StateDisconnected)
	{
    ConnectButton->SetLabel("Connect");
		ConnectIP->SetEnabled(true);
		ConnectPort->SetText("5038");
		ConnectPort->SetEnabled(true);
	}
	else
	{
		ConnectIP->SetEnabled(false);
		ConnectPort->SetEnabled(false);
	}		
	if (ConnectionState == StateConnected)
	{
    ConnectButton->SetLabel("Disconnect");
		OutText->MakeEditable(true);
	}
	else
	{
		OutText->MakeEditable(false);
	}
	return true;
}

void clsChatWindow::FrameResized(float width, float height)
{
	int32 ViewHeight = int((MainView->Bounds().Height() - TopClip) / 2 - 0.5);
	InTextView->ResizeTo(MainView->Bounds().Width(), ViewHeight);
	OutTextView->MoveTo(0, MainView->Bounds().Height() - ViewHeight );
	OutTextView->ResizeTo(MainView->Bounds().Width(), ViewHeight);
  InText->SetTextRect(BRect(2, 2, InText->Bounds().Width() - 2, 10));
  OutText->SetTextRect(BRect(2, 2, OutText->Bounds().Width() - 2, 10));
  BWindow::FrameResized(width, height);
}
