/*  IPTalk - A P2P Communications System for IP Networks
		Version 1.0
		(C) 2001 Kevin H. Patterson
		IPTalk Version 1.0 is distributed under the GNU General Public License.
		Please see the included file "GPL License.txt" for details.
*/

#include "clsListenWindow.h"

clsListenWindow::clsListenWindow()
:
	BWindow(BRect(400, 100, 580, 300), "IPTalk", B_TITLED_WINDOW, B_WILL_DRAW)
{
	Interfaces = new BList(2);
	ChatWindows = new BList(2);
	
	MainView = new BView(BWindow::Bounds(), "MainView", B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS);
	MainView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	BWindow::AddChild(MainView);
		
		NewSessionButton = new BButton(BRect(5, 5, 10, 10), "NewSessionButton", "New Session", new BMessage('NewS'), B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM);
		NewSessionButton->ResizeToPreferred();
	  NewSessionButton->MoveTo(MainView->Bounds().right - NewSessionButton->Bounds().Width() - 5, MainView->Bounds().bottom - NewSessionButton->Bounds().Height() - 5);
	  MainView->AddChild(NewSessionButton);
	  
		AboutButton = new BButton(BRect(5, 5, 10, 10), "AboutButton", "About", new BMessage('AbtB'), B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
		AboutButton->ResizeToPreferred();
	  AboutButton->MoveTo(MainView->Bounds().left + 5, MainView->Bounds().bottom - AboutButton->Bounds().Height() - 5);
	  MainView->AddChild(AboutButton);
	  
	  InterfacesView = new BView(BRect(0, 0, MainView->Bounds().Width(), 100), "InterfacesView", B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS);
	  InterfacesView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	  MainView->AddChild(InterfacesView);
	  
		InterfacesList = new BListView(BRect(5, 5, 10, 100), "InterfacesList", B_MULTIPLE_SELECTION_LIST, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE);
		{
		  BScrollView * InterfacesListScroll = new BScrollView("InterfacesListScroll", InterfacesList, B_FOLLOW_ALL_SIDES, B_WILL_DRAW | B_FRAME_EVENTS, false, true);
			InterfacesView->AddChild(InterfacesListScroll);
			InterfacesListScroll->MoveTo(5, 5);
			InterfacesListScroll->ResizeTo(InterfacesView->Bounds().Width() - 10, InterfacesView->Bounds().Height() - 10);
		}
		
		{
			NetEndpointA * NewInterface = new NetEndpointA(SOCK_STREAM);
			if (NewInterface->InitCheck() == B_ERROR)
				delete NewInterface;
			else
			{
				if (NewInterface->Bind(5038) != B_OK)
				{
					BAlert * MyAlert = new BAlert(
						"Network Error",
						"The specified port failed to bind to a network interface.\12BNetEndpoint::Bind()\12\12The specified port may be unavailable on the interface, or the interface may not be valid.",
						"Huh?",
						NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
						B_STOP_ALERT
					);
					MyAlert->Go();
					delete NewInterface;
				}
				else
				{
					if (NewInterface->Listen() != B_OK)
					{
						BAlert * MyAlert = new BAlert(
							"Network Error",
							"An attempt to listen on the specified port failed on a network interface.\12BNetEndpoint::Listen()\12\12What gives?!",
							"Help Me.",
							NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
							B_STOP_ALERT
						);
						MyAlert->Go();
						delete NewInterface;
					}
					else
					{
						Interfaces->AddItem(NewInterface);
						BString InterfaceName = NewInterface->LocalHost() << " port " << int(NewInterface->LocalPort());
						InterfacesList->AddItem(new BStringItem(InterfaceName.String()));
						NewInterface->Target = this;
						NewInterface->StartListening();
					}
				}
			}
		}
		
		bool InterfaceFailure = false;
		while (!InterfaceFailure)
		{
			NetEndpointA * NewInterface = new NetEndpointA(SOCK_STREAM);
			if (NewInterface->InitCheck() == B_ERROR)
			{
				InterfaceFailure = true;
				delete NewInterface;
			}
			else
			{
				if (NewInterface->Bind(5038) != B_OK)
				{
					InterfaceFailure = true;
					delete NewInterface;
				}
				else
				{
					if (NewInterface->Listen() != B_OK)
					{
						InterfaceFailure = true;
						BAlert * MyAlert = new BAlert(
							"Network Error",
							"An attempt to listen on the specified port failed on a network interface.\12BNetEndpoint::Listen()\12\12What gives?!",
							"Help Me.",
							NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
							B_WARNING_ALERT
						);
						MyAlert->Go();
						delete NewInterface;
					}
					else
					{
						Interfaces->AddItem(NewInterface);
						BString InterfaceName = NewInterface->LocalHost() << " port " << int(NewInterface->LocalPort());
						InterfacesList->AddItem(new BStringItem(InterfaceName.String()));
						NewInterface->Target = this;
						NewInterface->StartListening();
					}
				}
			}
		}
		
		//clsListenWindow::FrameResized(BWindow::Bounds().Width(), BWindow::Bounds().Height());
}

bool clsListenWindow::QuitRequested()
{
	NetEndpointA * CurrentInterface = NULL; 
	for (int32 i = 0; i < Interfaces->CountItems(); i++)
	{
		CurrentInterface = (NetEndpointA *)Interfaces->ItemAt(i);
		if (CurrentInterface)
		{
			CurrentInterface->StopListening();
		}
	}
	CurrentInterface = NULL;
// I could use some help cleaning this up...
/*
	clsChatWindow * CurrentSession = NULL; 
	for (int32 i = 0; i < ChatWindows->CountItems(); i++)
	{
		CurrentSession = (clsChatWindow*)ChatWindows->ItemAt(i);
		if (CurrentSession)
		{
			CurrentSession->Quit();
		}
	}
	CurrentSession = NULL;
*/	
	ChatWindows->RemoveItems(0, ChatWindows->CountItems());
	be_app->PostMessage(B_QUIT_REQUESTED);
	return BWindow::QuitRequested();
}

void clsListenWindow::MessageReceived(BMessage* Message)
{
	switch (Message->what)
	{
	case 'NewS':
		{
	  	clsChatWindow * NewSession = new clsChatWindow();
	  	ChatWindows->AddItem(NewSession);
	  	NewSession->Show();
	  }
		break;
	case 'InCn':
		{
			void * NewConnection = NULL;
			Message->FindPointer("connection", 0, &NewConnection);
	  	clsChatWindow * NewSession = new clsChatWindow((BNetEndpoint*)NewConnection);
	  	ChatWindows->AddItem(NewSession);
	  	NewSession->Show();
	  }
	  break;
	case 'AbtB':
		{
			BAlert * MyAlert = new BAlert(
				"About IPTalk",
				"IPTalk - A P2P Communications System for IP Networks\12"
				"Version 1.0\12"
				"(C) 2001 Kevin H. Patterson\12\12"
				"IPTalk Version 1.0 is distributed under the GNU General Public License. "
				"Please see the included files 'Readme.txt' and 'GPL License.txt' for details.",
				"Sure.",
				NULL, NULL, B_WIDTH_AS_USUAL, B_EVEN_SPACING,
				B_INFO_ALERT
			);
			MyAlert->Go();
		}	
	default:
	  BWindow::MessageReceived(Message);
	}
}

void clsListenWindow::FrameResized(float width, float height)
{
  BWindow::FrameResized(width, height);
}
